#include "stdafx.h"
#include "SimulationUtils.h"


SimulationUtils::SimulationUtils(void)
{
	srand ( time(NULL) );
	m_uni_dist = new boost::uniform_real<>(0,1);
	m_generator = new base_generator_type(42u);
	m_generator->seed(static_cast<unsigned int>(std::time(0)));
	m_uni = new boost::variate_generator<base_generator_type&, boost::uniform_real<> >(*m_generator, *m_uni_dist);
}



SimulationUtils::~SimulationUtils(void)
{
	delete m_uni_dist;
	delete m_generator;
	delete m_uni;
}

double SimulationUtils::GetUni()
{
	//return  ((double) rand() / (RAND_MAX+1)) ;
	return (*m_uni)();
}

double SimulationUtils::GestStdExpRandom(double lambda)
{
	//double u = ((double) rand() / (RAND_MAX+1)) ;
	double u = (*m_uni)();
	return -log(1-u)*(lambda);
}

double SimulationUtils::GetRelativeError(vector<double> S)
{
	double cnt = 0;
	double N = S.size();	
	return Std(S)/(sqrt(N)*Mean(S));
}

double SimulationUtils::Mean(vector<double> S)
{
	double res = 0;
	double N = S.size();
	for(int i=0;i<N; i++)
	{
		res+=S[i];		
	}
	res = res/N;
	return res;
}

double SimulationUtils::Std(vector<double> S)
{
	double l = Mean(S);
	double tmp = 0;
	double N = S.size();
	for(int i=0;i<N; i++)
	{
		tmp += (S[i]-l)*(S[i]-l);			
	}
	tmp = sqrt(tmp/N);
	return tmp;
}

void  SimulationUtils::StartCPUCount()
{
	m_begin_time = clock();
}

void  SimulationUtils::ResetCPUCount()
{
	StartCPUCount();
}

float SimulationUtils::GetCPUCount()
{
	return float( clock () - m_begin_time ) /  CLOCKS_PER_SEC;
}

double SimulationUtils::Factorial(double x)
{
	double fac = 1;
	if(x==0 || x==1) 
		return 1;
	else{
		while(x>1)
		{
			fac = fac*x;
			x = x-1;
		}
	}
	return fac;
}

double SimulationUtils::Bin(double m, double i)
{
	return Factorial(m)/(Factorial(i)*Factorial(m-i));
}