#pragma once

#include <iostream>
#include <ctime>            // std::time
#include <vector>
#include <algorithm>

#include <boost/random/mersenne_twister.hpp>
#include <boost/random/linear_congruential.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/random/variate_generator.hpp>

using namespace std;

class SimulationUtils
{
private:
	// This is a typedef for a random number generator.
	// Try boost::mt19937 or boost::ecuyer1988 instead of boost::minstd_rand
	typedef boost::mt19937 base_generator_type;
	boost::uniform_real<> *m_uni_dist;
	base_generator_type *m_generator;
	boost::variate_generator<base_generator_type&, boost::uniform_real<> > *m_uni;

	clock_t m_begin_time;

public:
	SimulationUtils(void);
	~SimulationUtils(void);

	// Get random Exp(lambda)
	
	double GestStdExpRandom(double lambda);
	double GetUni();

	double GetRelativeError(vector<double> S);
	double Mean(vector<double> S);
	double Std(vector<double> S);

	void StartCPUCount();
	void ResetCPUCount();
	float GetCPUCount(); 

	// Binomial
	double Bin(double m, double i);
	double Factorial(double x);
};

