// SpectraV2_GeneralCNF.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
#include <string>
#include <algorithm>
#include "MonoCNFInstance.h"
#include "SimulationUtils.h"


using namespace std;

vector<double> GetComullativeDistrF(vector<double>& spectra);
double Bin(int n,int k);

double EstimateMonoCover(int N, double p, MonoCNFInstance& instance);
int PerformSingleRunMC(vector<int>& perm, MonoCNFInstance& graph);

void PrintVector(vector<double>& spectra);

int main(int argc, char* argv[])
{
	cout<<"test spectra V2"<<endl;
	//string base_path = "C:\\Users\\Slava\\SkyDrive\\Slava\\Research\\vertex-covers\\SpectraIlyaJosepjSlava\\cover-spectra\\INSTANCES\\";
	string base_path = "C:\\Users\\user\\SkyDrive\\Slava\\Research\\vertex-covers\\SpectraIlyaJosepjSlava\\cover-spectra\\INSTANCES\\";
	
	//string path = base_path.append("star50.cnf");
	//string path = base_path.append("star150.cnf");
	//string path = base_path.append("star300.cnf");

	//string path = base_path.append("hyper4._cachet.cnf");
	//string path = base_path.append("hyper5._cachet.cnf");
	//string path = base_path.append("hyper6._cachet.cnf");
	//string path = base_path.append("hyper7._cachet.cnf");
	string path = base_path.append("hyper8._cachet.cnf");

	//string path = base_path.append("erdos100_05_cachet.cnf");
	//string path = base_path.append("erdos100_02_cachet.cnf");
	//string path = base_path.append("erdos100_01_cachet.cnf");
	//string path = base_path.append("erdos200_05_cachet.cnf");
	SimulationUtils utils;

	MonoCNFInstance instance;
	instance.Load(path);

	int N = 100000000;

	vector<double> res;
	vector<double> cputime;
	for (int i=0;i<100; i++)
	{
		utils.StartCPUCount();
		//double l = EstimateVC( N, 0.5, graph);
		double l = EstimateMonoCover( N, 0.5, instance);
		double cpu = utils.GetCPUCount();
		cout<<i<<" "<<l<<" "<< cpu<<endl;
		cputime.push_back(cpu);
		res.push_back(l);
	}
	cout<<"---------------------------------"<<endl;
	cout<<utils.Mean(res)<<" "<<utils.GetRelativeError(res)<<" "<<utils.Mean(cputime)<<endl;

	return 0;
}

void PrintVector(vector<double>& spectra)
{
	cout<<"---------------------------------"<<endl;
	for(int i=0; i<spectra.size(); i++)
		cout<<" "<<spectra[i];

	cout<<endl;
	cout<<"---------------------------------"<<endl;
}

double EstimateMonoCover(int N, double p, MonoCNFInstance& instance)
{
	int sz = instance.GetVarsNum();
	vector<int> vars; 
	for(int i=0;i<sz; i++)
		vars.push_back(i);

	vector<double> vec(sz+1);
	vec.assign(sz+1,0);


	for (int i=0;i<N; i++)
	{
		int ind = PerformSingleRunMC(vars,instance);
		vec[ind+1]++;
	}

	// normalize
	for(int i=0;i<vec.size(); i++)
	{
		vec[i]/=(double)N;
	}

	// calculate cdf
	vec = GetComullativeDistrF(vec);

	//PrintVector(vec);

	// get an estimator
	double l = 0;
	int n=vec.size()-1;
	for (int i=0; i<=n; i++)
	{
		l+=Bin(n,i)*vec[i];
	}

	return l;
}

int PerformSingleRunMC(vector<int>& perm, MonoCNFInstance& instance)
{
	random_shuffle(perm.begin(),perm.end());	
	int b = instance.GetCriticalIndex(perm);
	return b;
}

vector<double> GetComullativeDistrF(vector<double>& spectra)
{
	vector<double> ComF;
	ComF.push_back(spectra[0]);
	for(int i=1;i<spectra.size(); i++)
	{
		double prev = ComF[i-1];
		double newV = prev + spectra[i];
		ComF.push_back(newV);
	}

	return ComF;
}

double Bin(int n,int k)
{
	double res = 1;
	for (int i=1; i<= min(k,n-k); i++)
	{
		res = res*(n-i+1);
		res = res/i;
	}
	return res;
}